local addon = _G.LFGTimer;

local config = addon:NewModule("Config");

addon.Config = config;

local L = LibStub("AceLocale-3.0"):GetLocale("LFGTimer");
local db;

function config:OnInitialize()
	db = addon.Vars.profile;
	
	addon.Vars.RegisterCallback(self, "OnProfileChanged", "UpdateDB");
	addon.Vars.RegisterCallback(self, "OnProfileCopied", "UpdateDB");
	addon.Vars.RegisterCallback(self, "OnProfileReset", "UpdateDB");
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("LFGTimer", self.Options);
	
	self.Options.args.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.Vars);
end

function config:UpdateDB(event, database)
	db = database.profile;
end

function config:Show()
	LibStub("AceConfigDialog-3.0"):Open("LFGTimer");
end

config.Options = {
	type = "group",
	order = 1,
	name = "LFG Timer",
	childGroups = 'tree',
	get = function(info)
		return db[ info[#info] ];
	end,
	set = function(info, value)
		db[ info[#info] ] = value;
		addon:UpdateFrameArt();
		addon:UpdateFrameShown();
	end,
	args = {
		FrameDisplay = {
			type = 'group',
			order = 1,
			name = L["Frame options"],
			childGroups = 'tab',
			disabled = function()
				return not db.Frame.Show;
			end,
			get = function(info)
				return db.Frame[ info[#info] ];
			end,
			set = function(info, value)
				db.Frame[ info[#info] ] = value;
				addon:UpdateFrameArt();
				addon:UpdateFrameShown();
			end,
			args = {
				General = {
					type = 'group',
					order = 1,
					name = L["General"],
					desc = L["General frame options."],
					args = {
						Show = {
							type = 'toggle',
							order = 1,
							name = L["Show frame"],
							desc = L["Show a queue status frame."],
							disabled = false,
						},
						Locked = {
							type = 'toggle',
							order = 2,
							name = L["Lock the frame"],
							desc = L["Lock the frame in place, preventing it from being moved."],
						},
						Type = {
							type = 'select',
							order = 3,
							name = L["Frame type"],
							desc = L["Type of queue status frame to show.\n\n|cffffd200Queue status clone|r: Clone of the Blizzard Queue Status frame. This is a graphical frame.\n\n|cffffd200Text tooltip|r: Customizable tooltip that can show detailed information about queues. This IS NOT a graphical frame."],
							values = {
								clone = L["Queue status clone"],
								custom = L["Text tooltip"],
							},
						},
					},
				},
				ShowWhen = {
					type = 'group',
					order = 2,
					name = L["Show when"],
					desc = L["Control when the queue status frame is shown."],
					args = {
						States = {
							type = 'group',
							order = 1,
							guiInline = true,
							name = L["By queue status"],
							get = function(info)
								return db.Frame.Queue.States[ info[#info] ];
							end,
							set = function(info, value)
								db.Frame.Queue.States[ info[#info] ] = value;
								addon:UpdateFrameArt();
								addon:UpdateFrameShown();
							end,
							args = {
								notQueued = {
									type = 'toggle',
									order = 1,
									name = L["Not queued"],
									desc = L["Show the queue status frame when not queued for an activity."],
								},
								proposal = {
									type = 'toggle',
									order = 2,
									name = L["Queue available"],
									desc = L["Show the queue status frame when an active queue prompts the user to accept."],
								},
								inParty = {
									type = 'toggle',
									order = 3,
									name = L["In a group"],
									desc = L["Show the queue status frame when the user is in an instance group or battleground raid."],
								},
								queuedNoData = {
									type = 'toggle',
									order = 3,
									name = L["Acquiring queue data"],
									desc = L["Show the queue status frame when queued but before relavent queue data is available."],
								},
								queued = {
									type = 'toggle',
									order = 3,
									name = L["In a queue"],
									desc = L["Show the queue status frame while queued."],
								},
							},
						},
						Types = {
							type = 'group',
							order = 2,
							guiInline = true,
							name = L["By queue type"],
							get = function(info)
								return db.Frame.Queue.Types[ info[#info] ];
							end,
							set = function(info, value)
								db.Frame.Queue.Types[ info[#info] ] = value;
								addon:UpdateFrameArt();
								addon:UpdateFrameShown();
							end,
							args = {
								LookingForDungeon = {
									type = 'toggle',
									order = 1,
									name = L["Looking for Dungeon"],
									desc = L["Show queue information for Looking for Dungeon queues. This includes random, specific and/or heroic dungeon queues but does not include Raid Finder or Scenario queues."],
								},
								RaidFinder = {
									type = 'toggle',
									order = 2,
									name = L["Raid Finder"],
									desc = L["Show queue information for Raid Finder queues."]
								},
								Scenarios = {
									type = 'toggle',
									order = 3,
									name = L["Scenarios"],
									desc = L["Show queue information for a Scenario queues. This includes random or specific scenario queues but does not include Raid Finder or Looking for Dungeon queues."],
								},
								PvP = {
									type = 'toggle',
									order = 5,
									name = L["Battleground & Arena"],
									desc = L["Show queue information for Battleground and Arena queues. This includes Rated Battleground and Arena Skirmish queues. but does not include World PvP queues."]
								},
								WorldPvP = {
									type = 'toggle',
									order = 6,
									name = L["World PvP"],
									desc = L["Show queue information for World PvP queues. This includes Wintergrasp and Tol Barad."]
								},
							},
						},
					},
				},
				Data = {
					type = 'group',
					order = 3,
					name = L["Tooltip data"],
					desc = L["Control what information the Text Tooltip frame displays."],
					disabled = function()
						return not db.Frame.Show or db.Frame.Type ~= 'custom';
					end,
					childGroups = 'select',
					args = {
						LookingForDungeon = {
							type = 'group',
							order = 1,
							name = L["Looking for Dungeon"],
							args = {
								QueueTimes = {
									type = 'group',
									order = 1,
									guiInline = true,
									name = L["Queue times"],
									get = function(info)
										return db.Frame.Queue.Data.LookingForDungeon.QueueTimes[ info[#info] ];
									end,
									set = function(info, value)
										db.Frame.Queue.Data.LookingForDungeon.QueueTimes[ info[#info] ] = value;
										addon:UpdateFrameArt();
										addon:UpdateFrameShown();
									end,
									args = {
										Tank = {
											type = 'toggle',
											order = 1,
											name = L["Tank queue times"],
											desc = L["Show queue time information for the Tank role."],
										},
										Healer = {
											type = 'toggle',
											order = 1,
											name = L["Healer queue times"],
											desc = L["Show queue time information for the Healer role."],
										},
										DPS = {
											type = 'toggle',
											order = 1,
											name = L["DPS queue times"],
											desc = L["Show queue time information for the DPS role."],
										},
										Self = {
											type = 'toggle',
											order = 1,
											name = L["Your queue times"],
											desc = L["Show queue time information for your group."],
										},
									},
								},
								Party = {
									type = 'group',
									order = 1,
									guiInline = true,
									name = L["Party members found"],
									get = function(info)
										return db.Frame.Queue.Data.LookingForDungeon.Party[ info[#info] ];
									end,
									set = function(info, value)
										db.Frame.Queue.Data.LookingForDungeon.Party[ info[#info] ] = value;
										addon:UpdateFrameArt();
										addon:UpdateFrameShown();
									end,
									args = {
										Tank = {
											type = 'toggle',
											order = 1,
											name = L["Tanks found"],
											desc = L["Show information on the number of Tanks found versus the total needed."],
										},
										Healer = {
											type = 'toggle',
											order = 2,
											name = L["Healers found"],
											desc = L["Show information on the number of Healers found versus the total needed."],
										},
										DPS = {
											type = 'toggle',
											order = 3,
											name = L["DPS found"],
											desc = L["Show information on the number of DPS found versus the total needed."],
										},
										Total = {
											type = 'toggle',
											order = 4,
											name = L["Party members"],
											desc = L["Show information on the number of party members found versus the total needed."],
										},
									},
								},
							},
						},
						RaidFinder = {
							type = 'group',
							order = 2,
							name = L["Raid Finder"],
							args = {
								QueueTimes = {
									type = 'group',
									order = 1,
									guiInline = true,
									name = L["Queue times"],
									get = function(info)
										return db.Frame.Queue.Data.RaidFinder.QueueTimes[ info[#info] ];
									end,
									set = function(info, value)
										db.Frame.Queue.Data.RaidFinder.QueueTimes[ info[#info] ] = value;
										addon:UpdateFrameArt();
										addon:UpdateFrameShown();
									end,
									args = {
										Tank = {
											type = 'toggle',
											order = 1,
											name = L["Tank queue times"],
											desc = L["Show queue time information for the Tank role."],
										},
										Healer = {
											type = 'toggle',
											order = 1,
											name = L["Healer queue times"],
											desc = L["Show queue time information for the Healer role."],
										},
										DPS = {
											type = 'toggle',
											order = 1,
											name = L["DPS queue times"],
											desc = L["Show queue time information for the DPS role."],
										},
										Self = {
											type = 'toggle',
											order = 1,
											name = L["Your queue times"],
											desc = L["Show queue time information for your group."],
										},
									},
								},
								Party = {
									type = 'group',
									order = 1,
									guiInline = true,
									name = L["Party members found"],
									get = function(info)
										return db.Frame.Queue.Data.RaidFinder.Party[ info[#info] ];
									end,
									set = function(info, value)
										db.Frame.Queue.Data.RaidFinder.Party[ info[#info] ] = value;
										addon:UpdateFrameArt();
										addon:UpdateFrameShown();
									end,
									args = {
										Tank = {
											type = 'toggle',
											order = 1,
											name = L["Tanks found"],
											desc = L["Show information on the number of Tanks found versus the total needed."],
										},
										Healer = {
											type = 'toggle',
											order = 2,
											name = L["Healers found"],
											desc = L["Show information on the number of Healers found versus the total needed."],
										},
										DPS = {
											type = 'toggle',
											order = 3,
											name = L["DPS found"],
											desc = L["Show information on the number of DPS found versus the total needed."],
										},
										Total = {
											type = 'toggle',
											order = 4,
											name = L["Party members"],
											desc = L["Show information on the number of party members found versus the total needed."],
										},
									},
								},
							},
						},
						Scenarios = {
							type = 'group',
							order = 2,
							name = L["Scenarios"],
							args = {
								QueueTimes = {
									type = 'group',
									order = 1,
									guiInline = true,
									name = L["Queue times"],
									get = function(info)
										return db.Frame.Queue.Data.Scenarios.QueueTimes[ info[#info] ];
									end,
									set = function(info, value)
										db.Frame.Queue.Data.Scenarios.QueueTimes[ info[#info] ] = value;
										addon:UpdateFrameArt();
										addon:UpdateFrameShown();
									end,
									args = {
										Tank = {
											type = 'toggle',
											order = 1,
											name = L["Tank queue times"],
											desc = L["Show queue time information for the Tank role."],
										},
										Healer = {
											type = 'toggle',
											order = 1,
											name = L["Healer queue times"],
											desc = L["Show queue time information for the Healer role."],
										},
										DPS = {
											type = 'toggle',
											order = 1,
											name = L["DPS queue times"],
											desc = L["Show queue time information for the DPS role."],
										},
										Self = {
											type = 'toggle',
											order = 1,
											name = L["Your queue times"],
											desc = L["Show queue time information for your group."],
										},
									},
								},
								Party = {
									type = 'group',
									order = 1,
									guiInline = true,
									name = L["Party members found"],
									get = function(info)
										return db.Frame.Queue.Data.Scenarios.Party[ info[#info] ];
									end,
									set = function(info, value)
										db.Frame.Queue.Data.Scenarios.Party[ info[#info] ] = value;
										addon:UpdateFrameArt();
										addon:UpdateFrameShown();
									end,
									args = {
										Tank = {
											type = 'toggle',
											order = 1,
											name = L["Tanks found"],
											desc = L["Show information on the number of Tanks found versus the total needed."],
										},
										Healer = {
											type = 'toggle',
											order = 2,
											name = L["Healers found"],
											desc = L["Show information on the number of Healers found versus the total needed."],
										},
										DPS = {
											type = 'toggle',
											order = 3,
											name = L["DPS found"],
											desc = L["Show information on the number of DPS found versus the total needed."],
										},
										Total = {
											type = 'toggle',
											order = 4,
											name = L["Party members"],
											desc = L["Show information on the number of party members found versus the total needed."],
										},
									},
								},
							},
						},
						PvP = {
							type = 'group',
							order = 5,
							name = L["Player vs Player"],
							get = function(info)
								return db.Frame.Queue.Data.PvP[ info[#info] ];
							end,
							set = function(info, value)
								db.Frame.Queue.Data.PvP[ info[#info] ] = value;
										addon:UpdateFrameArt();
										addon:UpdateFrameShown();
							end,
							args = {
								TeamSize = {
									type = 'toggle',
									order = 2,
									name = L["Team size"],
									desc = L["Show the team size for the selected PvP queue.\n\n|cffffd200Note|r: Some battleground queues always report the team size as zero (0)."],
								},
								WaitTime = {
									type = 'toggle',
									order = 3,
									name = L["Wait time"],
									desc = L["Show the estimated wait time."],
								},
							},
						},
						WorldPvP = {
							type = 'group',
							order = 5,
							name = L["World PvP"],
							get = function(info)
								return db.Frame.Queue.Data.WorldPvP[ info[#info] ];
							end,
							set = function(info, value)
								db.Frame.Queue.Data.WorldPvP[ info[#info] ] = value;
								addon:UpdateFrameArt();
								addon:UpdateFrameShown();
							end,
							args = {
								TimeLeft = {
									type = 'toggle',
									order = 1,
									name = L["Time left"],
									desc = L["Show the time remaining for the selected World PvP battle.\n\n|cffffd200Note|r: Some World PvP queues may not support this value."],
								},
							},
						},
					},
				},
				Art = {
					type = 'group',
					order = 4,
					name = L["Artwork and text"],
					desc = L["Control how the Text Tooltip frame looks."],
					childGroups = 'tab',
					disabled = function()
						return not db.Frame.Show or db.Frame.Type ~= 'custom';
					end,
					args = {
						Backdrop = {
							type = 'group',
							order = 2,
							name = L["Backdrop"],
							desc = L["Background and border options"],
							get = function(info)
								if info.type == 'color' then
									local t = db.Frame.Backdrop[ info[#info] ];
									return t.r, t.g, t.b, t.a;
								else
									return db.Frame.Backdrop[ info[#info] ];
								end
							end,
							set = function(info, ...)
								if info.type == 'color' then
									local t = db.Frame.Backdrop[ info[#info] ];
									t.r, t.g, t.b, t.a = ...;
								else
									db.Frame.Backdrop[ info[#info] ] = ...;
								end
								addon:UpdateFrameArt();
							end,
							args = {
								Background = {
									type = 'select',
									order = 1,
									name = L["Background texture"],
									desc = L["Texture used as the tooltip's background."],
									dialogControl = 'LSM30_Background',
									values = AceGUIWidgetLSMlists.background,
									width = 'full',
								},
								BackgroundColor = {
									type = 'color',
									order = 2,
									name = L["Background texture color"],
									desc = L["Color used to tint the tooltip's background texture.\n|cffffd200Note|r: Darker textures are less effected by this option."],
									width = 'full',
								},
								Tile = {
									type = 'toggle',
									order = 3,
									name = L["Tile the background texture"],
									desc = L["Repeat the background texture horizontally and vertically."],
									width = 'full',
								},
								TileSize = {
									type = 'range',
									order = 4,
									name = L["Background tile size"],
									desc = L["Size of the blocks used when tiling the background texture."],
									width = 'full',
									min = 2,
									max = 64,
									step = 1,
									bigStep = 2,
									disabled = function(info)
										return not db.Frame.Backdrop.Tile;
									end,
								},
								Inset = {
									type = 'range',
									order = 5,
									name = L["Background inset"],
									desc = L["Distance between the border and the background texture."],
									width = 'full',
									min = 0,
									max = 32,
									step = 1,
									bigStep = 2,
								},
								Border = {
									type = 'select',
									order = 7,
									name = L["Border texture"],
									desc = L["Texture used as the tooltip's border."],
									dialogControl = 'LSM30_Border',
									values = AceGUIWidgetLSMlists.border,
									width = 'full',
								},
								BorderColor = {
									type = 'color',
									order = 8,
									name = L["Border texture color"],
									desc = L["Color used to tint the tooltip's border texture.\n|cffffd200Note|r: Darker textures are less effected by this option."],
									width = 'full',
								},
								EdgeSize = {
									type = 'range',
									order = 9,
									name = L["Border size"],
									desc = L["Thickness of the border texture."],
									width = 'full',
									min = 2,
									max = 32,
									step = 1,
									bigStep = 2,
								},
							},
						},
						Text = {
							type = 'group',
							order = 3,
							name = L["Text"],
							desc = L["Text appearance options"],
							get = function(info)
								if info.type == 'color' then
									local t = db.Frame.Text[ info[#info] ];
									return t.r, t.g, t.b, t.a;
								else
									return db.Frame.Text[ info[#info] ];
								end
							end,
							set = function(info, ...)
								if info.type == 'color' then
									local t = db.Frame.Text[ info[#info] ];
									t.r, t.g, t.b, t.a = ...;
								else
									db.Frame.Text[ info[#info] ] = ...;
								end
							end,
							args = {
								Font = {
									type = 'select',
									order = 1,
									name = L["Font"],
									desc = L["Font used for the tooltip text."],
									dialogControl = 'LSM30_Font',
									values = AceGUIWidgetLSMlists.font,
									width = 'full',
								},
								Size = {
									type = 'range',
									order = 2,
									name = L["Text size"],
									desc = L["Font size used for the tooltip text."],
									width = 'full',
									min = 4,
									max = 20,
									step = 1,
								},
								Outline = {
									type = 'select',
									order = 3,
									name = L["Text outline"],
									desc = L["Add an outline to the tooltip text."],
									width = 'full',
									values = {
										[" "] = L["No outline"],
										["OUTLINE"] = L["Thin outline"],
										["THICKOUTLINE"] = L["Thick outline"],
									},
								},
								Color = {
									type = 'color',
									order = 4,
									name = L["Text color"],
									desc = L["Color used for the tooltip text."],
									width = 'full',
								},
							},
						},
					},
				},
			},
		},
	},
};
